/* jshint node: true */
'use strict';

let BasePlugin           = require('ember-cli-deploy-plugin')
let fs                   = require('fs-promise')
let path                 = require('path')
let RSVP                 = require('rsvp')
let AWS                  = require('aws-sdk')
let Lambda               = new AWS.Lambda({ region: 'us-east-1' })
let UpdateLambdaFunc     = RSVP.denodeify(Lambda.updateFunctionCode.bind(Lambda))
let GetLambdaAlias       = RSVP.denodeify(Lambda.getAlias.bind(Lambda))
let CreateLambdaAlias    = RSVP.denodeify(Lambda.createAlias.bind(Lambda))
let UpdateLambdaAlias    = RSVP.denodeify(Lambda.updateAlias.bind(Lambda))
let PublishLambdaVersion = RSVP.denodeify(Lambda.publishVersion.bind(Lambda))
let exec                 = RSVP.denodeify(require('child_process').exec)
let Promise              = RSVP.Promise

module.exports = {
  name: 'ember-cli-deploy-lambda',

  createDeployPlugin(opts) {
    let DeployPlugin = BasePlugin.extend({
      name: opts.name,

      requiredConfig: ['function-name', 'lambda-dist-dir'],

      defaultConfig: {
        "lambda-dist-dir": 'tmp/lambda-dist'
      },

      didBuild(context) {
        let self = this
        let packageSkeletonPath = path.join(__dirname, 'assets', 'lambda-package')
        let lambdaDistDir = this.readConfig('lambda-dist-dir')

        return fs.copy(packageSkeletonPath, lambdaDistDir)
          .then(_ => self._copyServerFiles())
          .then(_ => self._copyIndexHtml(context))
          .then(_ => self._npmInstallForLambdaDist())
          .then(_ => { return { lambdaDistDir } })
      },

      willUpload(context) {
        let lambdaDistDir = this.readConfig('lambda-dist-dir')
        let zipFile = `${path.basename(lambdaDistDir)}.zip`
        let lambdaDistZip = `tmp/${zipFile}`

        this.log(`zipping up ${lambdaDistDir}`, { verbose: true })

        return exec(`zip -qr ${zipFile} *`, { cwd: lambdaDistDir })
          .then(_ => {
            let srcZip = path.join(lambdaDistDir, zipFile)

            return fs.move(srcZip , lambdaDistZip, { clobber: true })
          })
          .then(_ => { return { lambdaDistZip }})
      },

      upload(context) {
        let self = this

        return fs.readFile(context.lambdaDistZip)
          .then(fileBuf => self._uploadToLambda(fileBuf))
          .then(_ => self._publishVersion())
          .then(version => self._tagVersion(version, context))
          .then(lambdaRevisionKey => { return { lambdaRevisionKey } })
      },

      didDeploy(context) {
        let revisionKey = context.lambdaRevisionKey

        this.log(`Deployed but did not activate revision ${revisionKey}. To activate, run: ember deploy:activate ${context.deployTarget} --revision=${revisionKey}\n`)
      },

      activate(context) {
        let self            = this
        let Name            = `${context.deployTarget}-current`
        let FunctionVersion = context.commandOptions.revision
        let FunctionName    = this.readConfig('function-name')

        return GetLambdaAlias({ Name, FunctionName })
          .catch(err => {
            if (err.code === 'ResourceNotFoundException') {
              return CreateLambdaAlias({ Name, FunctionName, FunctionVersion })
            } else {
              return Promise.reject(err)
            }
          })
          .then(_ => UpdateLambdaAlias({ Name, FunctionName, FunctionVersion }))
          .then(_ => self.log(`✔ Activated revision \`${FunctionVersion}\``))
      },

      _getVersionForAlias(alias) {
        let FunctionName = this.readConfig('function-name')

        return GetLambdaAlias({ FunctionName: FunctionName, Name: alias })
          .then(resp => resp.FunctionVersion)
      },

      _publishVersion() {
        let FunctionName = this.readConfig('function-name')

        return PublishLambdaVersion({ FunctionName })
          .then(resp => resp.Version)
      },

      _tagVersion(version, context) {
        let FunctionName    = this.readConfig('function-name')
        let FunctionVersion = version
        let timestamp       = Math.floor(Date.now() / 1000)
        let Name            = `${context.deployTarget}-${timestamp}`

        return CreateLambdaAlias({ FunctionName, FunctionVersion, Name })
          .then(_ => version)
      },

      _uploadToLambda(fileBuf) {
        let lambdaFunc = this.readConfig('function-name')

        return UpdateLambdaFunc({
          FunctionName: lambdaFunc,
          ZipFile: fileBuf
        })
      },

      _npmInstallForLambdaDist() {
        let lambdaDistDir = this.readConfig('lambda-dist-dir')
        let serverRoutesDir = path.join(lambdaDistDir, 'server-routes')

        return exec("npm install --production", { cwd: lambdaDistDir })
          .then(_ => exec("npm install --production", { cwd: serverRoutesDir }));
      },

      _copyIndexHtml(context) {
        let src = path.join(context.distDir, 'index.html')
        let dest = path.join(this.readConfig('lambda-dist-dir'), 'index.html')

        return fs.copy(src, dest)
      },

      _copyServerFiles(context) {
        let lambdaDistDir = this.readConfig('lambda-dist-dir')

        return fs.copy('server-routes', path.join(lambdaDistDir, 'server-routes'))
      }
    })

    return new DeployPlugin()
  }
};
